@@ -4,5 +4,7 @@
 # Please see the file COPYING in this distribution for usage
 # terms.
 
-from model import Layer, State, Model
-from params import Params
+from .model import Layer, State, Model
+from .param import Params
+from .misc import *
+from glimpse.models.base import BuildLayer
