@@ -1,16 +1,38 @@
 """Miscellaneous functions for dealing with numpy arrays."""
 
-# Copyright (c) 2011 Mick Thomure
+# Copyright (c) 2011-2013 Mick Thomure
 # All rights reserved.
 #
-# Please see the file COPYING in this distribution for usage terms.
+# Please see the file LICENSE.txt in this distribution for usage terms.
 
 import Image
 import math
 import numpy as np
+import scipy.misc
 
-#: Element type for an array of Glimpse activation values.
-ACTIVATION_DTYPE = np.float32
+def fromimage(im, flatten = 0):
+  """Convert Image to numpy array."""
+  if isinstance(im, np.ndarray):
+    return im  # make function idempotent
+  if not Image.isImageType(im):
+    raise TypeError("Input is not a PIL image.")
+  if flatten:
+    im = im.convert('F')
+  elif im.mode == '1':
+    # Add workaround to bug on converting binary image to numpy array.
+    im = im.convert('L')
+  elif im.mode == 'LA':
+    raise ValueError("Greyscale with alpha channel not supported. Convert to"
+        " RGBA first.")
+  return np.array(im)
+
+def toimage(arr, *args, **kw):
+  if Image.isImageType(arr):
+    return arr
+  return scipy.misc.toimage(arr, *args, **kw)
+
+FromImage = fromimage
+ToImage = toimage
 
 def ArgMax(array):
   """Short-hand to find array indices containing the maximum value."""
@@ -29,51 +51,25 @@ def ScaleUnitNorm(x):
     x /= norm
   return x
 
-def ArrayToGreyscaleImage(array, normalize = True):
-  """Create a greyscale image from a 2D numpy array.
-
-  This function assumes range of input values contains 0.
-
-  """
-  if array.dtype != np.float32:
-    array = array.astype(np.float32)
-  if len(array.shape) > 2:
-    # Stack bands vertically.
-    array = array.reshape((-1, array.shape[-1]))
-  if normalize:
-    # Normalize array values to lie in [0, 255].
-    max_val = max(abs(array.min()), array.max())
-    # Map values to lie in [-.5, .5]
-    array = array / (2 * max_val)  # makes copy of input
-    # Map to [0, 1]
-    array += 0.5
-    # Map to [0, 255]
-    array *= 255
-  # Convert to unsigned chars
-  array = np.asarray(array, dtype = np.uint8)
-  return Image.fromarray(array, 'L')
-
-def ArrayToRGBImage(array):
-  """Create a color image from an array.
-
-  :rtype: PIL.Image
-
-  """
-  return Image.fromarray(array, 'RGB')
-
 def FlattenLists(data):
   """Flatten nested lists to a single list.
 
+  Any arrays are returned unchanged. Thus, this function can be used to turn a
+  list of lists (of lists...) of arrays into a single list of arrays.
+
   :param data: Input data.
-  :type data: N-dimensional list
+  :type data: N-dimensional iterable
   :returns: Flattened copy of input data.
   :rtype: list
 
   """
   # From: http://stackoverflow.com/questions/2158395
   result = []
+  if isinstance(data, np.ndarray):
+    return data
   for el in data:
-    if hasattr(el, "__iter__") and not isinstance(el, basestring):
+    if hasattr(el, "__iter__") and not (isinstance(el, basestring) or \
+        isinstance(el, dict) or isinstance(el, np.ndarray)):
       result.extend(FlattenLists(el))
     else:
       result.append(el)
@@ -91,9 +87,11 @@ def FlattenArrays(data):
 
   """
   data = FlattenLists(data)
-  assert len(data) > 0
+  if len(data) == 0:
+    return np.empty((0,))
   dtype = data[0].dtype
-  assert all(x.dtype == dtype for x in data)
+  if not all(x.dtype == dtype for x in data):
+    raise ValueError("All arrays must have the same dtype")
   out_size = sum(x.size for x in data)
   out = np.empty((out_size,), dtype)
   offset = 0
@@ -107,9 +105,22 @@ def FlattenArrays(data):
 ArrayListToVector = FlattenArrays
 
 def PadArray(data, out_shape, cval):
-  """Pad the border of an array with a constant value."""
+  """Pad the border of an array with a constant value.
+
+  :param data: Input data.
+  :type data: N-dim ndarray
+  :param out_shape: Dimensions of output array.
+  :type out_shape: list of int
+  :param cval: Value to use for border region of output array.
+  :rtype: N-dim ndarray
+  :return: Padded input data.
+
+  """
   out_shape = np.array(out_shape)
   in_shape = np.array(data.shape)
+  # Ensure the requested size is at least as large as the input array
+  for i in range(len(out_shape)):
+    out_shape[i] = max(out_shape[i], in_shape[i])
   result = np.empty(out_shape)
   result[:] = cval
   begin = ((out_shape - in_shape) / 2.0).astype(int)
@@ -120,11 +131,11 @@ def CropArray(data, out_shape):
   """Remove the border of an array.
 
   :param data: Input array.
-  :type data: ND ndarray
+  :type data: N-dim ndarray
   :param out_shape: Shape of central region to return. If length is less than
      `data.shape`, this is assumed to specify the range in the last axes.
   :type out_shape: list of int
-  :rtype: ND ndarray
+  :rtype: N-dim ndarray
   :returns: View of the central region of the input array.
 
   """
