@@ -1,15 +1,15 @@
 """Functions for dealing with the kernel matrices of linear filters."""
 
-# Copyright (c) 2011 Mick Thomure
+# Copyright (c) 2011-2013 Mick Thomure
 # All rights reserved.
 #
-# Please see the file COPYING in this distribution for usage terms.
+# Please see the file LICENSE.txt in this distribution for usage terms.
 
-from garray import ScaleUnitNorm, ACTIVATION_DTYPE
-from gimage import ImageToArray
 import math
 import numpy as np
 
+from .garray import ScaleUnitNorm, fromimage
+
 def MakeGaborKernel(kwidth, theta, gamma = 0.6, sigma = None, phi = 0,
     lambda_ = None, scale_norm = True):
   """Create a kernel matrix by evaluating the 2-D Gabor function.
@@ -73,8 +73,6 @@ def MakeGaborKernels(kwidth, num_orientations, num_phases, shift_orientations,
      edge on a black background, and vice versa).
   :param bool shift_orientations: Whether to rotate Gabor through a small angle
      (a value of True helps compensate for aliasing).
-  :param bool scale_norm: If true, then the kernel vector is scaled to have unit
-     norm.
   :returns: Kernel arrays indexed by orientation and phase.
   :rtype: 4D ndarray of float
 
@@ -96,7 +94,7 @@ def MakeGaborKernels(kwidth, num_orientations, num_phases, shift_orientations,
   ks = [[ MakeGaborKernel(kwidth, theta, phi = phi, **args)
       for phi in phis ]
     for theta in thetas ]
-  ks = np.array(ks, ACTIVATION_DTYPE)
+  ks = np.array(ks)
   return ks
 
 def MakeMultiScaleGaborKernels(kwidth, num_scales, num_orientations, num_phases,
@@ -151,7 +149,7 @@ def MakeMultiScaleGaborKernels(kwidth, num_scales, num_orientations, num_phases,
         for phi in phis ]
       for theta in thetas ]
     for sigma, lambda_ in zip(scales, lambdas) ]
-  ks = np.array(ks, ACTIVATION_DTYPE)
+  ks = np.array(ks)
   return ks
 
 # gaussian scale is:
@@ -217,7 +215,7 @@ def DrawGaborAsLine(orient, num_orientations = 8, kwidth = 11,
   draw = ImageDraw.Draw(im)
   draw.line((x1 + hw, y1 + hw, x2 + hw, y2 + hw), fill = 255,
       width = line_width)
-  data = ImageToArray(im) / 255
+  data = fromimage(im) / 255
   data = np.rot90(data)
   return data
 
@@ -254,6 +252,35 @@ def MakeRandomKernels(nkernels, kshape, normalize = True, mean = 0,
       k /= np.linalg.norm(k)
   return kernels
 
+def MakeUniformRandomKernels(nkernels, kshape, normalize = True, low = 0, high = 1):
+  """Sample kernel arrays from multivariate uniform distribution.
+
+  :param int nkernels: Number of kernels to create.
+  :param kshape: Dimensions of each kernel.
+  :type kshape: list of int
+  :param bool normalize: Whether the resulting kernels should be scaled to have
+     unit norm.
+  :param float low: Lower bound on sampled values (inclusive).
+  :param float high: Upper bound on sampled values (exclusive).
+  :returns: Set of kernel arrays.
+  :rtype: N-dimensional ndarray of float, where `N = len(kshape)+1`
+
+  Examples:
+
+  >>> nkernels = 10
+  >>> kshape = (8, 5, 5)
+  >>> kernels = MakeUniformRandomKernels(nkernels, kshape)
+  >>> assert(kernels.shape[0] == nkernels)
+  >>> assert(kernels.shape[1:] == kshape)
+
+  """
+  shape = (nkernels,) + tuple(kshape)
+  kernels = np.random.uniform(low, high, shape)
+  if normalize:
+    for kernel in kernels:
+      kernel /= np.linalg.norm(kernel)
+  return kernels
+
 def MakeLanczosKernel():
   """Construct a smoothing filter based on the Lanczos window.
 
@@ -278,3 +305,69 @@ def MakeLanczosKernel():
   k1 = sinc(X) * sinc(X / float(a))
   k2 = outer(k1, k1)
   return k2
+
+def Blackman1d(n, alpha = 0.16):
+  """The 1-dimensional Blackman window.
+
+  .. deprecated:: 0.1
+     Use :func:`numpy.blackman` instead.
+
+  :param int n: Number of elements.
+  :param float alpha: The parameter of the Blackman window.
+  :rtype: 1D ndarray
+
+  """
+  a0 = (1 - alpha) / 2.0
+  a1 = 0.5
+  a2 = alpha / 2.0
+  x = np.arange(n)
+  return a0 - a1 * np.cos(2 * math.pi * x / (n - 1)) + \
+      a2 * np.cos(4 * math.pi * x / (n - 1))
+
+def Blackman2d(ny, nx, power = 1):
+  """The 2-dimensional Blackman window.
+
+  :param int ny: Number of elements along the Y-axis.
+  :param int nx: Number of elements along the X-axis.
+  :param float power: Elongates the X-direction (if greater than 1), or shortens
+     it (if less than 1).
+  :rtype: 2D ndarray
+
+  """
+  a = np.empty([ny, nx])
+  bx = Blackman1d(nx)
+  bx = np.maximum(bx, np.zeros_like(bx)) ** power
+  by = Blackman1d(ny)
+  a[:] = bx
+  a = (a.T * by).T
+  return np.maximum(a, np.zeros_like(a))
+
+def Gabor(sigma, theta, phi, gamma, lambda_, kernel):
+  """Fill a 2D matrix using values of the Gabor function.
+
+  :param float sigma: Variance of the Gaussian function.
+  :param float theta: Orientation of the parallel stripes of the Gabor function.
+  :param float phi: Phase offset (match a black edge on on white background, or
+     vice versa).
+  :param float gamma: Spatial aspect ratio. set to 1.0 to get circular variance
+     on the Gaussian function, and set to less than one to get elongated
+     variance.
+  :param float lambda_: Wavelength of sinusoidal function.
+  :param kernel: Array in which to store kernel values (must not be None).
+  :type kernel: 2D ndarray of float
+  :returns: The *kernel* parameter.
+  :rtype: 2D ndarray of float
+
+  """
+  height, width = kernel.shape
+  size = height * width
+  for j in range(height):
+    for i in range(width):
+      y = j - height / 2.0
+      x = i - width / 2.0
+      yo = -1.0 * x * math.sin(theta) + y * math.cos(theta)
+      xo = x * math.cos(theta) + y * math.sin(theta)
+      kernel[j,i] = math.exp(-1.0 *
+                      (xo**2 + gamma**2 * yo**2) / (2.0 * sigma**2)) * \
+                    math.sin(phi + 2 * math.pi * xo / lambda_)
+  return kernel
