@@ -8,11 +8,10 @@ ZeroMQ sockets.
 
 """
 
-# Copyright (c) 2011 Mick Thomure
+# Copyright (c) 2011-2013 Mick Thomure
 # All rights reserved.
 #
-# Please see the file COPYING in this distribution for usage
-# terms.
+# Please see the file LICENSE.txt in this distribution for usage terms.
 
 import itertools
 import logging
@@ -21,11 +20,11 @@ import socket
 import time
 import zmq
 
-def SocketTypeToString(type):
+def SocketTypeToString(type_):
   """Get a textual representation of a socket type ID."""
   mapping = { zmq.PUSH : 'PUSH', zmq.PULL : 'PULL',
       zmq.PUB : 'PUB', zmq.SUB : 'SUB', zmq.REQ : 'REQ', zmq.REP : 'REP' }
-  return mapping[type]
+  return mapping[type_]
 
 class ReceiverTimeoutException(Exception):
   """Indicates that a ZMQ recv() command timed out."""
@@ -216,8 +215,6 @@ class BasicVentilator(object):
     logging.info("BasicVentilator: starting send")
     num_requests = 0
     for request in requests:
-#      logging.info("BasicVentilator: sending task %d at time %s" % \
-#          (self.num_total_requests, time.time()))
       self.sender.send_pyobj(request)
       self.num_total_requests += 1
       num_requests += 1
